@@ -5,10 +5,6 @@
 #### Simplified
 ##########################
 
-# Random Data Generator
-.rdg <- function() s %.!% 'rdg()'
-
-
 
 # Adjusted Rand Index
 adj.rand.index <- function(c1,c2) {
@@ -54,17 +50,12 @@ mass <- function(...,fixed=TRUE) {
 }
 
 # Not exported:  Mass wrapper
-.mass <- function(mass) s %.!% '
-  Mass(R.evalD0(mass+"$value"))
-'
+.mass <- function(mass) s$Mass(mass$value)
 
-.massFactory <- function(mass) s %.!% '
-  if ( R.evalL0(mass+"$fixed")  ) {
-    Mass.factory(R.evalD0(mass+"$value"))
-  } else {
-    Mass.factory(R.evalD0(mass+"$shape"),R.evalD0(mass+"$rate"),rdg())
-  }
-'
+.massFactory <- function(mass) {
+  if ( mass$fixed ) s$Mass.factory(mass$value)
+  else s$Mass.factory(mass$shape,mass$rate,s$rdg())
+}
 
 print.shallot.mass <- function(x, ...) {
   if ( x$fixed ) cat("mass fixed at ",x$value,"\n",sep="")
@@ -103,17 +94,12 @@ discount <- function(...,fixed=TRUE) {
 }
 
 # Not exported:  Discount wrapper 
-.discount <- function(discount) s %.!% '
-  Discount(R.evalD0(discount+"$value"))
-'
+.discount <- function(discount) s$Discount(discount$value)
 
-.discountFactory <- function(discount) s %.!% '
-  if ( R.evalL0(discount+"$fixed")  ) {
-    Discount.factory(R.evalD0(discount+"$value"))
-  } else {
-    Discount.factory(R.evalD0(discount+"$shape1"),R.evalD0(discount+"$shape2"),rdg())
-  }
-'
+.discountFactory <- function(discount) {
+  if ( discount$fixed ) s$Discount.factory(discount$value)
+  else s$Discount.factory(discount$shape,discount$rate,s$rdg())
+}
 
 print.shallot.discount <- function(x, ...) {
   if ( x$fixed ) cat("discount fixed at ",x$value,"\n",sep="")
@@ -150,14 +136,14 @@ permutation <- function(...,n.items=NULL,fixed=TRUE) {
 }
 
 .permutation <- function(permutation) {
-  s$.Permutation$apply(permutation$value-1L)
+  s$.Permutation.apply(permutation$value-1L)
 }
 
 .permutationFactory <- function(permutation) {
   if ( permutation$fixed ) {
-    s$.Permutation$factory(permutation$value-1L)
+    s$.Permutation.factory(permutation$value-1L)
   } else {
-    s$.Permutation$factory(permutation$n.items,.rdg())
+    s$.Permutation.factory(permutation$n.items,s$rdg())
   }
 }
 
@@ -262,23 +248,23 @@ decay.generic <- function(temperature,distance,type,max.temperature,max.distance
 
 .decay <- function(decay) {
   temp <- min(decay$temperature$value,decay$max.temperature)
-       if ( decay$type == "reciprocal" )  s$.decay.ReciprocalDecay$new(temp)
-  else if ( decay$type == "exponential" ) s$.decay.ExponentialDecay$new(temp)
-  else if ( decay$type == "subtraction" ) s$.decay.SubtractionDecay$new(temp,decay$max.distance)
+       if ( decay$type == "reciprocal" )  s$.new_decay.ReciprocalDecay(temp)
+  else if ( decay$type == "exponential" ) s$.new_decay.ExponentialDecay(temp)
+  else if ( decay$type == "subtraction" ) s$.new_decay.SubtractionDecay(temp,decay$max.distance)
 }
 
 .decayFactory <- function(decay) {
   if ( decay$temperature$fixed ) {
     temp <- min(decay$temperature$value,decay$max.temperature)
-         if ( decay$type == "reciprocal" )  s$.decay.ReciprocalDecayFactory$factory(temp)
-    else if ( decay$type == "exponential" ) s$.decay.ExponentialDecayFactory$factory(temp)
-    else if ( decay$type == "subtraction" ) s$.decay.SubtractionDecayFactory$new(decay$max.distance)$factory(temp)
+         if ( decay$type == "reciprocal" )  s$.decay.ReciprocalDecayFactory.factory(temp)
+    else if ( decay$type == "exponential" ) s$.decay.ExponentialDecayFactory.factory(temp)
+    else if ( decay$type == "subtraction" ) s$.decay.SubtractionDecayFactory.new(decay$max.distance)$factory(temp)
   } else {
     shape <- decay$temperature$shape
     rate <- decay$temperature$rate
-         if ( decay$type == "reciprocal" )  s$.decay.ReciprocalDecayFactory$factory(shape,rate,.rdg())
-    else if ( decay$type == "exponential" ) s$.decay.ExponentialDecayFactory$factory(shape,rate,.rdg())
-    else if ( decay$type == "subtraction" ) s$.decay.SubtractionDecayFactory$new(decay$max.distance)$factory(shape,rate,.rdg())
+         if ( decay$type == "reciprocal" )  s$.decay.ReciprocalDecayFactory.factory(shape,rate,s$rdg())
+    else if ( decay$type == "exponential" ) s$.decay.ExponentialDecayFactory.factory(shape,rate,s$rdg())
+    else if ( decay$type == "subtraction" ) s$.new_decay.SubtractionDecayFactory(decay$max.distance)$factory(shape,rate,s$rdg())
   }
 }
 
@@ -317,37 +303,37 @@ attraction <- function(permutation, decay) {
 }
 
 .distance <- function(distance) {
-  s$.Distance$apply(as.matrix(distance),FALSE)
+  s$.Distance.apply(as.matrix(distance),FALSE)
 }
 
 .attraction <- function(attraction) {
   if ( attraction$constant ) {
-    s$.Attraction$constant(attraction$n.items)
+    s$.Attraction.constant(attraction$n.items)
   } else {
     tryCatch(
-      s$.Attraction$apply(.distance(as.matrix(attraction$decay$distance)),.permutation(attraction$permutation),.decay(attraction$decay))
+      s$.Attraction.apply(.distance(as.matrix(attraction$decay$distance)),.permutation(attraction$permutation),.decay(attraction$decay))
     ,error = function(e) stop("Attraction is invalid because 'distance' and 'decay' appear to be incompatible.  Perhaps lower the 'temperature'."))
   }
 }
 
 .attractionFactory <- function(attraction) {
   if ( attraction$constant ) {
-    s$.Attraction$factory(attraction$n.items)
+    s$.Attraction.factory(attraction$n.items)
   } else if ( attraction$permutation$fixed && attraction$decay$temperature$fixed ) {
     tryCatch(
-      s$.Attraction$factory(.distance(as.matrix(attraction$decay$distance)),.permutation(attraction$permutation),.decay(attraction$decay))
+      s$.Attraction.factory(.distance(as.matrix(attraction$decay$distance)),.permutation(attraction$permutation),.decay(attraction$decay))
     ,error = function(e) stop("Attraction is invalid because 'distance' and 'decay' appear to be incompatible.  Perhaps lower the 'temperature'."))
   } else if ( attraction$permutation$fixed ) {
     tryCatch(
-      s$.Attraction$factory(.distance(as.matrix(attraction$decay$distance)),.permutation(attraction$permutation),.decayFactory(attraction$decay))
+      s$.Attraction.factory(.distance(as.matrix(attraction$decay$distance)),.permutation(attraction$permutation),.decayFactory(attraction$decay))
     ,error = function(e) stop("Attraction is invalid because 'distance' and 'decay' appear to be incompatible.  Perhaps lower the 'temperature'."))
   } else if ( attraction$decay$temperature$fixed ) {
     tryCatch(
-      s$.Attraction$factory(.distance(as.matrix(attraction$decay$distance)),.permutationFactory(attraction$permutation),.decay(attraction$decay))
+      s$.Attraction.factory(.distance(as.matrix(attraction$decay$distance)),.permutationFactory(attraction$permutation),.decay(attraction$decay))
     ,error = function(e) stop("Attraction is invalid because 'distance' and 'decay' appear to be incompatible.  Perhaps lower the 'temperature'."))
   } else {
     tryCatch(
-      s$.Attraction$factory(.distance(as.matrix(attraction$decay$distance)),.permutationFactory(attraction$permutation),.decayFactory(attraction$decay))
+      s$.Attraction.factory(.distance(as.matrix(attraction$decay$distance)),.permutationFactory(attraction$permutation),.decayFactory(attraction$decay))
     ,error = function(e) stop("Attraction is invalid because 'distance' and 'decay' appear to be incompatible.  Perhaps lower the 'temperature'."))
   }
 }
@@ -494,15 +480,15 @@ nsubsets.random <- function(x,n.samples) {
   mass <- .massFactory(x$mass)
   n.items <- x$n.items
   if ( inherits(x,"shallot.distribution.ewens") ) {
-    s$.Ewens$sampleNumberOfSubsets(n.items,mass,n.samples)
+    s$Ewens.sampleNumberOfSubsets(n.items,mass,n.samples)
   } else if ( inherits(x,"shallot.distribution.ewensAttraction") ) {
-    s$.EwensAttraction$sampleNumberOfSubsets(n.items,mass,n.samples)
+    s$EwensAttraction.sampleNumberOfSubsets(n.items,mass,n.samples)
   } else {
     discount <- .discountFactory(x$discount)
     if ( inherits(x,"shallot.distribution.ewensPitman") ) {
-      s$.EwensPitman$sampleNumberOfSubsets(n.items,mass,discount,n.samples)
+      s$EwensPitman.sampleNumberOfSubsets(n.items,mass,discount,n.samples)
     } else if ( inherits(x,"shallot.distribution.ewensPitmanAttraction" ) ) {
-      s$.EwensPitmanAttraction$sampleNumberOfSubsets(n.items,mass,discount,n.samples)
+      s$EwensPitmanAttraction.sampleNumberOfSubsets(n.items,mass,discount,n.samples)
     } else stop("Unrecognized distribution.")
   }
 }
@@ -519,16 +505,16 @@ nsubsets.probability <- function(x,n.subsets) {
   mass <- .mass(x$mass)
   n.items <- x$n.items
   if ( inherits(x,"shallot.distribution.ewens") ) {
-    s$.Ewens$probabilityNumberOfSubsets(n.items,n.subsets,mass)
+    s$Ewens.probabilityNumberOfSubsets(n.items,n.subsets,mass)
   } else if ( inherits(x,"shallot.distribution.ewensAttraction") ) {
-    s$.EwensAttraction$probabilityNumberOfSubsets(n.items,n.subsets,mass)
+    s$EwensAttraction.probabilityNumberOfSubsets(n.items,n.subsets,mass)
   } else {
     if ( ! x$discount$fixed ) stop("'discount' must be fixed for this function, but emperical estimates are available through the 'nsubsets.random' function.")
     discount <- .discount(x$discount)
     if ( inherits(x,"shallot.distribution.ewensPitman") ) {
-      s$.EwensPitman$probabilityNumberOfSubsets(n.items,n.subsets,mass,discount)
+      s$EwensPitman.probabilityNumberOfSubsets(n.items,n.subsets,mass,discount)
     } else if ( inherits(x,"shallot.distribution.ewensPitmanAttraction") ) {
-      s$.EwensPitmanAttraction$probabilityNumberOfSubsets(n.items,n.subsets,mass,discount)
+      s$EwensPitmanAttraction.probabilityNumberOfSubsets(n.items,n.subsets,mass,discount)
     } else stop("Unrecognized distribution.")
   }
 }
@@ -544,16 +530,16 @@ nsubsets.average <- function(x) {
   mass <- .mass(x$mass)
   n.items <- x$n.items
   if ( inherits(x,"shallot.distribution.ewens") ) {
-    s$.Ewens$meanNumberOfSubsets(n.items,mass)
+    s$Ewens.meanNumberOfSubsets(n.items,mass)
   } else if ( inherits(x,"shallot.distribution.ewensAttraction") ) {
-    s$.EwensAttraction$meanNumberOfSubsets(n.items,mass)
+    s$EwensAttraction.meanNumberOfSubsets(n.items,mass)
   } else {
     if ( ! x$discount$fixed ) stop("'discount' must be fixed for this function, but emperical estimates are available through the 'nsubsets.random' function.")
     discount <- .discount(x$discount)
     if ( inherits(x,"shallot.distribution.ewensPitman") ) {
-      s$.EwensPitman$meanNumberOfSubsets(n.items,mass,discount)
+      s$EwensPitman.meanNumberOfSubsets(n.items,mass,discount)
     } else if ( inherits(x,"shallot.distribution.ewensPitmanAttraction") ) {
-      s$.EwensPitmanAttraction$meanNumberOfSubsets(n.items,mass,discount)
+      s$EwensPitmanAttraction.meanNumberOfSubsets(n.items,mass,discount)
     } else stop("Unrecognized distribution.")
   }
 }
@@ -569,9 +555,9 @@ nsubsets.variance <- function(x) {
   mass <- .mass(x$mass)
   n.items <- x$n.items
   if ( inherits(x,"shallot.distribution.ewens") ) {
-    s$.Ewens$varianceNumberOfSubsets(n.items,mass)
+    s$Ewens.varianceNumberOfSubsets(n.items,mass)
   } else if ( inherits(x,"shallot.distribution.ewensAttraction") ) {
-    s$.EwensAttraction$varianceNumberOfSubsets(n.items,mass)
+    s$EwensAttraction.varianceNumberOfSubsets(n.items,mass)
   } else {
     if ( ! x$discount$fixed ) stop("'discount' must be fixed, but emperical estimates are available through the 'nsubsets.random' function.")
     discount <- .discount(x$discount)
@@ -590,10 +576,10 @@ nsubsets.variance <- function(x) {
 # Sample for partition distributions.
 .ewens <- function(x, samplingModel=.nullModel()) {
   mass <- .mass(x$mass)
-  s$.Ewens$apply(samplingModel,mass,.AS.REFERENCE=TRUE)
+  s$.Ewens.apply(samplingModel,mass)
 }
 
-.sample.ewens <- function(nItems=0L, massFactory=scalaNull('() => Mass')) s %.!% '
+.sample.ewens <- function(nItems, massFactory) s(nItems=as.integer(nItems[1]),massFactory=massFactory) ^ '
   val samplingModel = NullSamplingModel
   val partitionModelFactory = Ewens.factory(samplingModel,massFactory)
   PartitionModel.forwardSampler(nItems,partitionModelFactory)
@@ -602,10 +588,10 @@ nsubsets.variance <- function(x) {
 .ewensPitman <- function(x, samplingModel=.nullModel()) {
   mass <- .mass(x$mass)
   discount <- .discount(x$discount)
-  s$.EwensPitman$apply(samplingModel,mass,discount,.AS.REFERENCE=TRUE)
+  s$.EwensPitman.apply(samplingModel,mass,discount)
 }
 
-.sample.ewensPitman <- function(nItems=0L, massFactory=scalaNull('() => Mass'), discountFactory=scalaNull('() => Discount')) s %.!% '
+.sample.ewensPitman <- function(nItems, massFactory, discountFactory) s(nItems=as.integer(nItems[1]),massFactory=massFactory,discountFactory=discountFactory) ^ '
   val samplingModel = NullSamplingModel
   val partitionModelFactory = EwensPitman.factory(samplingModel,massFactory,discountFactory)
   PartitionModel.forwardSampler(nItems,partitionModelFactory)
@@ -614,10 +600,10 @@ nsubsets.variance <- function(x) {
 .ewensAttraction <- function(x, samplingModel=.nullModel()) {
   mass <- .mass(x$mass)
   attraction <- .attraction(x$attraction)
-  s$.EwensAttraction$apply(samplingModel,mass,attraction,.AS.REFERENCE=TRUE)
+  s$.EwensAttraction.apply(samplingModel,mass,attraction)
 }
 
-.sample.ewensAttraction <- function(nItems=0L, massFactory=scalaNull('() => Mass'), attractionFactory=scalaNull('() => Attraction')) s %.!% '
+.sample.ewensAttraction <- function(nItems, massFactory, attractionFactory) s(nItems=as.integer(nItems[1]),massFactory=massFactory,attractionFactory=attractionFactory) ^ '
   val samplingModel = NullSamplingModel
   val partitionModelFactory = EwensAttraction.factory(samplingModel,massFactory,attractionFactory)
   PartitionModel.forwardSampler(nItems,partitionModelFactory)
@@ -627,29 +613,28 @@ nsubsets.variance <- function(x) {
   mass <- .mass(x$mass)
   discount <- .discount(x$discount)
   attraction <- .attraction(x$attraction)
-  s$.EwensPitmanAttraction$apply(samplingModel,mass,discount,attraction,.AS.REFERENCE=TRUE)
+  s$.EwensPitmanAttraction.apply(samplingModel,mass,discount,attraction)
 }
 
-.sample.ewensPitmanAttraction <- function(nItems=0L, massFactory=scalaNull('() => Mass'), discountFactory=scalaNull('() => Discount'), attractionFactory=scalaNull('() => Attraction')) s %.!% '
+.sample.ewensPitmanAttraction <- function(nItems, massFactory, discountFactory, attractionFactory) s(nItems=as.integer(nItems[1]), massFactory=massFactory, discountFactory=discountFactory, attractionFactory=attractionFactory) ^ '
   val samplingModel = NullSamplingModel
   val partitionModelFactory = EwensPitmanAttraction.factory(samplingModel,massFactory,discountFactory,attractionFactory)
   PartitionModel.forwardSampler(nItems,partitionModelFactory)
 '
 
-.partitionsToMatrix <- function(x=scalaNull('List[Partition[PersistentReference]]')) s %!% '
+.partitionsToMatrix <- function(x) s(x=x) * '
   x.map(_.toLabels).toArray
 '
 
-.partitionsToMatrixWithParameters <- function(x=scalaNull('List[Partition[PersistentReference]]')) s %!% '
+.partitionsToMatrixWithParameters <- function(x) s(x=x) ^ '
   val labelsWithParameters = x.map(_.toLabelsWithParameters)
   val labels = labelsWithParameters.map(_._1).toArray
   val parameters = labelsWithParameters.map(_._2.map(_.toString)).toArray
   (labels, parameters)
 '
 
-.sampleForward <- function(nSamples, rdg=scalaNull('RDG'), sampler=scalaNull('Function2[Int, RDG, List[Partition[Null]]]'), parallel=TRUE) {
-  nSamples <- as.integer(nSamples)
-  s %.!% '
+.sampleForward <- function(nSamples, rdg, sampler, parallel=TRUE) {
+  s(nSamples=as.integer(nSamples), sampler=sampler, parallel=parallel) ^ '
     if (!parallel) sampler(nSamples, rdg)
     else {
       val nCores = Runtime.getRuntime.availableProcessors
@@ -672,7 +657,7 @@ sample.partitions <- function(x, n.draws, parallel=TRUE) {
   } else if ( inherits(x,"shallot.distribution.ewensPitmanAttraction") ) {
     .sample.ewensPitmanAttraction(x$n.items,.massFactory(x$mass),.discountFactory(x$discount),.attractionFactory(x$attraction))
   } else stop("Unrecognized partition distribution.")
-  ref <- .sampleForward(n.draws,.rdg(),forwardSampler,parallel)
+  ref <- .sampleForward(n.draws,s$rdg(),forwardSampler,parallel)
   structure(list(ref=ref, names=x$names), class="shallot.samples.raw")
 }
 
@@ -685,142 +670,6 @@ print.shallot.samples.full <- function(x, ...) {
 }
 
 
-
-# Posterior simulation via MCMC.
-sample.partitions.posterior <- function(partition, sampling.model, partition.model, n.draws, massRWSD=0.5, discountRWSD=0.1, k=min(length(partition),25), temperatureRWSD=0.5, progress.bar=interactive()) {
-  sampler <- function(p=scalaNull("Partition[PersistentReference]"),
-                      sm=scalaNull("SamplingModel[PersistentReference]"),
-                      pm=scalaNull("PartitionModel[PersistentReference]"),
-                      pmR=NULL,
-                      pmType="",
-                      rdg=scalaNull("RDG"),
-                      progressBar, showProgressBar=TRUE) s %.!% '
-    val nDraws = R.getI0("n.draws")
-    val k = R.evalI0("k")
-    val massRWSD = R.evalD0("massRWSD")
-    val discountRWSD = R.evalD0("discountRWSD")
-    val temperatureRWSD = R.evalD0("temperatureRWSD")
-    val updateMass = R.evalL0("identical(%s$mass$fixed,FALSE)".format(pmR))
-    val monitorMass = AcceptanceRateMonitor()
-    val (massShape, massRate) = if ( updateMass ) {
-      (R.evalD0(pmR+"$mass$shape"), R.evalD0(pmR+"$mass$rate"))
-    } else (0.0, 0.0)
-    val updateDiscount = R.evalL0("identical(%s$discount$fixed,FALSE)".format(pmR))
-    val monitorDiscount = AcceptanceRateMonitor()
-    val (discountShape1, discountShape2) = if ( updateDiscount ) {
-      ( R.evalD0(pmR+"$discount$shape1"), R.evalD0(pmR+"$discount$shape2"))
-    } else (0.0, 0.0)
-    val updatePermutation = R.evalL0("identical(%s$attraction$permutation$fixed,FALSE)".format(pmR))
-    val monitorPermutation = AcceptanceRateMonitor()
-    val updateTemperature = R.evalL0("identical(%s$attraction$decay$temperature$fixed,FALSE)".format(pmR))
-    val monitorTemperature = AcceptanceRateMonitor()
-    val (temperatureShape, temperatureRate) = if ( updateTemperature ) {
-      (R.evalD0(pmR+"$attraction$decay$temperature$shape"), R.evalD0(pmR+"$attraction$decay$temperature$rate"))
-    } else (0.0, 0.0)
-    abstract class Sampler {
-      protected var _partition = p
-      def partition = _partition
-      def hyperparameters: Array[Double]
-      def labels: Array[String]
-      def next(): Unit
-    }
-    val sampler: Sampler = pmType match {
-      case "org.ddahl.shallot.distribution.EwensPitmanAttraction[org.ddahl.rscala.PersistentReference]" => new Sampler {
-          var _distribution = pm.asInstanceOf[org.ddahl.shallot.distribution.EwensPitmanAttraction[org.ddahl.rscala.PersistentReference]]
-          def next() = {
-            _partition = AuxiliaryGibbsSampler(_partition, sm, _distribution, rdg)._1
-            if ( updateMass ) 
-              _distribution = monitorMass(MassSampler.gaussianRandomWalk(_distribution, _partition, massShape, massRate, massRWSD, rdg))
-            if ( updateDiscount ) {
-              _distribution = monitorDiscount(DiscountSampler.gaussianRandomWalk(
-                                  _distribution, _partition, discountShape1, discountShape2, discountRWSD, rdg))
-            }
-            if ( updatePermutation ) {
-              _distribution = monitorPermutation(PermutationSampler.update(_distribution, _partition, k, rdg, Set()))
-            }
-            if ( updateTemperature ) {
-              _distribution = monitorTemperature(TemperatureSampler.gaussianRandomWalk(
-                                  _distribution, _partition, temperatureShape, temperatureRate, temperatureRWSD, rdg))
-            }
-          }
-          def hyperparameters = Array(_distribution.mass.value, _distribution.discount.value, _distribution.attraction.decay.temperature,
-                                      monitorMass.rate, monitorDiscount.rate, monitorPermutation.rate, monitorTemperature.rate)
-          def labels = Array("mass","discount","temperature","rate.mass","rate.discount","rate.permutation","rate.temperature")
-        }
-      case "org.ddahl.shallot.distribution.EwensAttraction[org.ddahl.rscala.PersistentReference]" => new Sampler {
-          var _distribution = pm.asInstanceOf[org.ddahl.shallot.distribution.EwensAttraction[org.ddahl.rscala.PersistentReference]]
-          def next() = {
-            _partition = AuxiliaryGibbsSampler(_partition, sm, _distribution, rdg)._1
-            if ( updateMass ) 
-              _distribution = monitorMass(MassSampler.gaussianRandomWalk(_distribution, _partition, massShape, massRate, massRWSD, rdg))
-            if ( updatePermutation ) {
-              _distribution = monitorPermutation(PermutationSampler.update(_distribution, _partition, k, rdg, Set()))
-            }
-            if ( updateTemperature ) {
-              _distribution = monitorTemperature(TemperatureSampler.gaussianRandomWalk(
-                                  _distribution, _partition, temperatureShape, temperatureRate, temperatureRWSD, rdg))
-            }
-          }
-          def hyperparameters = Array(_distribution.mass.value, _distribution.attraction.decay.temperature,
-                                      monitorMass.rate, monitorPermutation.rate, monitorTemperature.rate)
-          def labels = Array("mass","temperature","rate.mass","rate.permutation","rate.temperature")
-        }
-      case "org.ddahl.shallot.distribution.EwensPitman[org.ddahl.rscala.PersistentReference]" => new Sampler {
-          var _distribution = pm.asInstanceOf[org.ddahl.shallot.distribution.EwensPitman[org.ddahl.rscala.PersistentReference]]
-          def next() = {
-            _partition = AuxiliaryGibbsSampler(_partition, sm, _distribution, rdg)._1
-            if ( updateMass ) 
-              _distribution = monitorMass(MassSampler.gaussianRandomWalk(_distribution, _partition, massShape, massRate, massRWSD, rdg))
-            if ( updateDiscount ) {
-              _distribution = monitorDiscount(DiscountSampler.gaussianRandomWalk(
-                                  _distribution, _partition, discountShape1, discountShape2, discountRWSD, rdg))
-            }
-          }
-          def hyperparameters = Array(_distribution.mass.value, _distribution.discount.value, monitorMass.rate, monitorDiscount.rate)
-          def labels = Array("mass","discount","rate.mass","rate.discount")
-        }
-      case "org.ddahl.shallot.distribution.Ewens[org.ddahl.rscala.PersistentReference]" => new Sampler {
-          var _distribution = pm.asInstanceOf[org.ddahl.shallot.distribution.Ewens[org.ddahl.rscala.PersistentReference]]
-          def next() = {
-            _partition = AuxiliaryGibbsSampler(_partition, sm, _distribution, rdg)._1
-            if ( updateMass ) 
-              _distribution = monitorMass(MassSampler.gaussianRandomWalk(_distribution, _partition, massShape, massRate, massRWSD, rdg))
-          }
-          def hyperparameters = Array(_distribution.mass.value, monitorMass.rate)
-          def labels = Array("mass","rate.mass")
-        }
-      case _ => throw new IllegalArgumentException("Unknown partition distribution type.")
-    }
-    val samplesPartition = new scala.collection.mutable.ListBuffer[Partition[PersistentReference]]()
-    val samplesHyperparameters = new scala.collection.mutable.ListBuffer[Array[Double]]()
-    var counter = 0
-    for ( i <- 1 to nDraws ) {
-      sampler.next() // AuxiliaryGibbsSampler(partition, sm, pm, rdg)._1
-      samplesPartition       += sampler.partition
-      samplesHyperparameters += sampler.hyperparameters
-      if ( showProgressBar && ( 100*i % nDraws == 0 ) ) {
-        counter += 1
-        R.invoke("setTxtProgressBar",progressBar,counter)
-      }
-    }
-    (samplesPartition.toList, samplesHyperparameters.toArray, sampler.labels)
-  '
-  sm <- .samplingModel(sampling.model)
-  pm <- .partitionModel(partition.model, sm)
-  p <- .labels2partition(partition, sm)
-  rdg <- .rdg()
-  pb <- if ( progress.bar ) txtProgressBar(min=0, max=100, style=3) else NULL
-  full <- sampler(p,sm,pm,partition.model,pm$type,rdg,pb,progress.bar)
-  if ( progress.bar ) close(pb)
-  raw <- structure(list(ref=full$"_1"(), names=partition.model$names), class="shallot.samples.raw")
-  hyperparameters <- full$"_2"()
-  colnames(hyperparameters) <- full$"_3"()
-  hyperparameters <- as.data.frame(hyperparameters)
-  structure(list(raw=raw, hyperparameters=hyperparameters), class="shallot.samples.full")
-}
-
-
-
 .partitionModel <- function(x, samplingModel=.nullModel()) {
   if ( inherits(x,"shallot.distribution.ewens") ) {
     .ewens(x, samplingModel)
@@ -838,7 +687,7 @@ sample.partitions.posterior <- function(partition, sampling.model, partition.mod
 # Make probability mass function.
 partition.pmf <- function(x) {
   distribution <- .partitionModel(x)
-  pmf <- distribution$logProbability(scalaNull("Partition[PersistentReference]"),.EVALUATE=FALSE)
+  pmf <- distribution$logProbability
   function(x, log=TRUE) {
     partition <- if ( is.vector(x) ) .labels2partition(x,.nullModel())
     else if ( is.list(x) ) .partition2partition(x)
@@ -852,15 +701,16 @@ partition.pmf <- function(x) {
 
 # Serialize partitions to R.
 serializePartitions <- function(ref, names, as.matrix, sample.parameter) {
-  withParameters <- ( ! is.null(sample.parameter) ) && ( ref$type != "List[org.ddahl.shallot.parameter.partition.Partition[Null]]" )
+  withParameters <- ( ! is.null(sample.parameter) ) && ( ref[['type']] != "List[org.ddahl.shallot.parameter.partition.Partition[Null]]" )
   if ( withParameters ) {
     zandp <- .partitionsToMatrixWithParameters(ref)
     z <- zandp$"_1"()
     colnames(z) <- names
     p <- if ( withParameters && is.function(sample.parameter) ) apply(z,1,function(zz) lapply(1:max(zz),function(i) sample.parameter()))
     else {
-      extractor <- function(x=scalaNull("Array[Array[String]]"),i=0L) s %!% 'x(i-1)'
+      extractor <- function(x,i=0L) s(x=x) * 'x(i-1)'
       pp <- zandp$"_2"()
+      n.draws <- nrow(z)
       ppp <- vector(mode="list", length=n.draws)
       for ( i in 1:n.draws ) {
         ppp[[i]] <- sapply(extractor(pp,i), function(x) s$var(x), USE.NAMES=FALSE)
@@ -896,47 +746,6 @@ serializePartitions <- function(ref, names, as.matrix, sample.parameter) {
 
 
 
-# Sampling model
-sampling.model <- function(sample.parameter, log.density) {
-  if ( ! is.function(sample.parameter) ) stop("'sample.parameter' should be a function.")
-  if ( length(formals(sample.parameter)) != 2 ) stop("'sample.parameter' should take two arguments named 'indices' and 'parameter'.")
-  if ( ! identical(names(formals(sample.parameter)),c("indices","parameter")) )
-    stop("'sample.parameter' should take two arguments named 'indices' and 'parameter'.")
-  if ( ! is.function(log.density) ) stop("'log.density' should be a function.")
-  if ( length(formals(log.density)) != 3 ) stop("'log.density' should take three arguments named 'i', 'indices', and 'parameter'.")
-  if ( ! identical(names(formals(log.density)),c("i","indices","parameter")) )
-    stop("'log.density' should take three arguments named 'i', 'indices', and 'parameter'.")
-  tryCatch(p <- sample.parameter(), error=function(e) stop("'sample.parameter' should sample from the centering distribution when no arguments are provided."))
-  tryCatch(d <- log.density(1,1,p), error=function(e) stop("'log.density' should doesn't pass sanity check log.density(1,1,p)."))
-  if ( ( ! is.vector(d) ) || ( length(d) != 1 ) ) stop("'log.density' should return a numeric vector of length one.")
-  r <- list(sample.parameter=sample.parameter, log.density=log.density)
-  structure(r, class="shallot.distribution.data")
-}
-
-.samplingModel <- function(samplingModel) {
-  s %.!% '
-    val sp = R.evalReference(samplingModel+"$sample.parameter")
-    val ld = R.evalReference(samplingModel+"$log.density")
-
-    new SamplingModel[PersistentReference] {
-
-      def logDensity(i: Int, subset: Subset[PersistentReference]): Double = {
-        R.invokeD0(ld, i+1, subset.toArray.map(_+1), subset.parameter)
-      }
-
-      def sample(subset: Subset[PersistentReference]): PersistentReference = {
-        R.invokeReference(sp, subset.toArray.map(_+1), subset.parameter)
-      }
-
-      def sample(): PersistentReference = {
-        R.invokeReference(sp)
-      }
-
-    }
-  '
-}
-
-
 
 # Process partitions that were sampled.
 process.samples <- function(x, as.matrix=TRUE, expand=FALSE, sample.parameter=FALSE) {
@@ -964,13 +773,13 @@ process.samples <- function(x, as.matrix=TRUE, expand=FALSE, sample.parameter=FA
 
 
 # Null sampling model
-.nullModel <- function() s %.!% 'new GeneralNullSamplingModel[PersistentReference]()'
+.nullModel <- function() s$'.new_GeneralNullSamplingModel[PersistentReference]'()
 
 
 
 # Pairwise Probabilities
 enumerate.partitions <- function(n.items, as.matrix=TRUE) {
-  ref <- s$.Partition$enumerate(.nullModel(),as.integer(n.items)[1])
+  ref <- s$.Partition.enumerate(.nullModel(),as.integer(n.items)[1])
   serializePartitions(ref, NULL, as.matrix=as.matrix, sample.parameter=NULL)
 }
 
@@ -981,7 +790,7 @@ pairwise.probabilities <- function(x, parallel=TRUE) {
   if ( inherits(x,"shallot.samples.full") ) x <- x$raw
   if ( ! inherits(x,"shallot.samples.raw") ) stop("'x' should be a result from the functions 'sample.partition' or 'sample.partition.posterior'.")
   start.time <- proc.time()
-  ref <- s$.PairwiseProbability$apply(x$ref,as.logical(parallel))
+  ref <- s$.PairwiseProbability.apply(x$ref,as.logical(parallel))
   result <- list(ref=ref,n.items=ref$nItems(),names=x$names,proc.time=proc.time()-start.time)
   structure(result, class="shallot.pairwiseProbability")
 }
@@ -1001,18 +810,16 @@ estimate.partition <- function(x, pairwise.probabilities=NULL, max.subsets=0, ma
   if ( inherits(x,"shallot.samples.full") ) x <- x$raw
   if ( ! inherits(x,"shallot.samples.raw") ) stop("'x' should be a result from the functions 'sample.partition' or 'sample.partition.posterior'.")
   if ( is.null(pairwise.probabilities) ) pairwise.probabilities <- pairwise.probabilities(x)
-  ref <- s$.MinBinder$apply(
+  ref <- s$.MinBinder.apply(
       pairwise.probabilities$ref, x$ref, as.integer(max.subsets), as.integer(max.scans), as.logical(parallel))
   structure(ref$toLabels(), names=x$names)
 }
 
 
-
-# Confidence
-.labels2partition <- function(partition, samplingModel=scalaNull("SamplingModel[PersistentReference]")) {
-  partition <- as.integer(partition)
-  s %.!% '
-    Partition(samplingModel,partition)
+# Misc
+.labels2partition <- function(partition, samplingModel) {
+  s(partition=as.integer(partition),samplingModel=samplingModel) ^ '
+    Partition((i: Int) => samplingModel.sample(),partition)
   '
 }
 
@@ -1020,102 +827,12 @@ estimate.partition <- function(x, pairwise.probabilities=NULL, max.subsets=0, ma
   stop("Not yet implemented.")
 }
 
-confidence <- function(pairwise.probabilities, partition) {
-  tmpObj <- pairwise.probabilities$ref$confidenceComputations(.labels2partition(partition,.nullModel()))
-  partition <- tmpObj$"_1"() + 1
-  names(partition) <- pairwise.probabilities$names
-  confidence <- tmpObj$"_2"()
-  names(confidence) <- names(partition)
-  confidence.matrix <- tmpObj$"_3"()
-  dimnames(confidence.matrix) <- list(1:nrow(confidence.matrix),1:ncol(confidence.matrix))
-  order <- tmpObj$"_4"() + 1L
-  names(order) <- names(partition)
-  exemplar <- tmpObj$"_5"() + 1L
-  names(exemplar) <- 1:length(exemplar)
-  result <- list(partition=partition,confidence=confidence,confidence.matrix=confidence.matrix,exemplar=exemplar,order=order,pairwise.probabilities=pairwise.probabilities)
-  class(result) <- "shallot.confidence"
-  result
-}
-
-
-
-# Confidence or pairs plot
-.rotateForConfidencePlot <- function(pp=scalaNull("PairwiseProbability"), order=integer()) s %!% '
-  val nItems = pp.nItems
-  val xx = Array.ofDim[Double](nItems, nItems)
-  for (i <- 0 until nItems) {
-    for (j <- 0 until nItems) {
-      xx(i)(nItems - j - 1) = pp(order(i) - 1, order(j) - 1)
-    }
-  }
-  xx
-'
-
-plot.shallot.confidence <- function(x, partition=NULL, data=NULL, show.labels=length(x$partition)<=50, ...) {
-  if ( ! is.null(data) ) {
-    if ( ! is.null(partition) ) stop("'partition' must be 'NULL' for pairs plot.")
-    i <- x$exemplar[x$partition]
-    c <- rainbow(length(x$exemplar))[x$partition]
-    panelFnc <- function(x0,y0,...) {
-      points(x0,y0,col=c,pch=19,...)
-      segments(x0,y0,x0[i],y0[i],col=c,...)
-      points(x0[x$exemplar],y0[x$exemplar],pch=22,bg="white",cex=2,...)
-    }
-    pairs(data,panel=panelFnc)
-    return(invisible())
-  }
-  if ( is.null(partition) ) {
-    partition <- x$partition
-    o <- x$order
-  } else {
-    o <- order(partition)
-  }
-  pm <- .rotateForConfidencePlot(x$pairwise.probabilities$ref,o)
-  n <- nrow(pm)
-  sizes <- rle(partition[o])$lengths
-  cuts <- cumsum(sizes)
-  centers <- ( c(0,cuts[-length(cuts)]) + cuts ) / 2
-  cuts <- cuts[-length(cuts)]
-  labels <- rle(partition[o])$values
-  if ( show.labels ) {
-    mymai <- c(1.5,0.5,0.5,1.5)
-    cexscale <- 0.85 * 50 / length(partition)
-  } else {
-    mymai <- c(0,0,0,0)
-    cexscale <- 1 * 50 / length(partition)
-  }
-  opar <- par(pty="s",mai=mymai)
-  colors <- topo.colors(200)
-  colors <- rev(heat.colors(200))
-  image(x=1:n,y=1:n,z=pm,axes=FALSE,xlab="",ylab="",col=colors)
-  box()
-  abline(v=cuts+0.5,lwd=3)
-  abline(h=n-cuts+0.5,lwd=3)
-  text(centers+0.5,n-centers+0.5,labels,cex=0.8*cexscale*sizes)
-  if ( show.labels ) {
-    axisLabels <- if ( is.null(names(partition)) ) o
-    else names(partition[o])
-    axis(4,1:length(partition),rev(axisLabels),las=2,cex.axis=0.8*cexscale)
-    axis(1,1:length(partition),axisLabels,las=2,cex.axis=0.8*cexscale)
-    nn <- length(colors)
-    bx <- par("usr")
-    bx.cx <- c(bx[1] - 1.6 * (bx[2] - bx[1]) / 50, bx[1] - 0.3 * (bx[2] - bx[1]) / 50)
-    bx.cy <- c(bx[3], bx[3])
-    bx.sy <- (bx[4] - bx[3]) / nn
-    xx <- rep(bx.cx, each=2)
-    for ( i in 1:nn ) {
-      yy <- c(bx.cy[1] + (bx.sy * (i - 1)),
-              bx.cy[1] + (bx.sy * (i)),
-              bx.cy[1] + (bx.sy * (i)),
-              bx.cy[1] + (bx.sy * (i - 1)))
-      polygon(xx,yy,col=colors[i],border=colors[i],xpd=TRUE)
-    }
-  }
-  par(opar)
-  invisible()
-}
-
 latest <- function() {
+  install.packages('https://dahl.byu.edu/public/rscala_LATEST.tar.gz',repos=NULL,type='source')
+  install.packages('https://dahl.byu.edu/public/commonsMath_LATEST.tar.gz',repos=NULL,type='source')
+  install.packages('https://dahl.byu.edu/public/sdols_LATEST.tar.gz',repos=NULL,type='source')
   install.packages('https://dahl.byu.edu/public/shallot_LATEST.tar.gz',repos=NULL,type='source')
+  cat("\n\n*******\n\nPlease restart your R session.\n\n*******\n")
+
 }
 
